@@ -1,14 +1,18 @@
+import os
 import re
 import logging
 import asyncio
+from urllib.parse import urlparse
+
 import aiohttp
+from aiohttp import web
 import json
 
 from functools import partialmethod
 from . chat import Chat, Sender
 
 __author__ = "Stepan Zastupov"
-__copyright__ = "Copyright 2015, Stepan Zastupov"
+__copyright__ = "Copyright 2015-2017 Stepan Zastupov"
 __license__ = "MIT"
 
 API_URL = "https://api.telegram.org"
@@ -18,10 +22,12 @@ RETRY_CODES = [429, 500, 502, 503, 504]
 BOTAN_URL = "https://api.botan.io/track"
 
 MESSAGE_TYPES = [
-    "location", "photo", "document", "audio", "voice", "sticker", "contact"
+    "location", "photo", "document", "audio", "voice", "sticker", "contact",
+    "venue", "video", "game", "contact", "delete_chat_photo", "new_chat_photo",
+    "delete_chat_photo", "new_chat_member", "left_chat_member",
+    "new_chat_title"
 ]
 
-
 logger = logging.getLogger("aiotg")
 
 
@@ -43,6 +49,8 @@ class Bot:
         self.api_timeout = api_timeout
         self.botan_token = botan_token
         self.name = name
+        self.webhook_url = None
+        self.session = aiohttp.ClientSession()
 
         def no_handle(mt):
             return lambda chat, msg: logger.debug("no handle for %s", mt)
@@ -53,8 +61,14 @@ class Bot:
         self._inline = lambda iq: None
         self._callback = lambda c, cq: None
 
-    @asyncio.coroutine
-    def loop(self):
+    def __del__(self):
+        try:
+            self.session.close()
+        except:
+            # 😶
+            pass
+
+    async def loop(self):
         """
         Return bot's main loop as coroutine. Use with asyncio.
 
@@ -70,7 +84,7 @@ class Bot:
         """
         self._running = True
         while self._running:
-            updates = yield from self.api_call(
+            updates = await self.api_call(
                 'getUpdates',
                 offset=self._offset + 1,
                 timeout=self.api_timeout
@@ -79,12 +93,13 @@ class Bot:
 
     def run(self):
         """
-        Convenience method for running bots
+        Convenience method for running bots in getUpdates mode
 
         :Example:
 
         >>> if __name__ == '__main__':
         >>>     bot.run()
+
         """
         loop = asyncio.get_event_loop()
         try:
@@ -92,6 +107,37 @@ class Bot:
         except KeyboardInterrupt:
             self.stop()
 
+    def run_webhook(self, webhook_url, **options):
+        """
+        Convenience method for running bots in webhook mode
+
+        :Example:
+
+        >>> if __name__ == '__main__':
+        >>>     bot.run_webhook(webhook_url="https://yourserver.com/webhooktoken")
+
+        Additional documentation on https://core.telegram.org/bots/api#setwebhook
+        """
+        loop = asyncio.get_event_loop()
+        loop.run_until_complete(self.set_webhook(webhook_url, **options))
+        if webhook_url:
+            url = urlparse(webhook_url)
+            app = self.create_webhook_app(url.path, loop)
+            port = int(os.environ.get('PORT', 0)) or url.port
+            web.run_app(app, port=port)
+
+    def stop_webhook(self):
+        """
+        Use to switch from Webhook to getUpdates mode
+        """
+        self.run_webhook(webhook_url="")
+
+    def add_command(self, regexp, fn):
+        """
+        Manually register regexp based command
+        """
+        self._commands.append((regexp, fn))
+
     def command(self, regexp):
         """
         Register a new command
@@ -105,7 +151,7 @@ class Bot:
         >>>     return chat.reply(match.group(1))
         """
         def decorator(fn):
-            self._commands.append((regexp, fn))
+            self.add_command(regexp, fn)
             return fn
         return decorator
 
@@ -202,16 +248,16 @@ class Bot:
         url = "{0}/bot{1}/{2}".format(API_URL, self.api_token, method)
         logger.debug("api_call %s, %s", method, params)
 
-        response = await aiohttp.post(url, data=params)
+        response = await self.session.post(url, data=params)
 
         if response.status == 200:
-            return (await response.json())
+            return await response.json()
         elif response.status in RETRY_CODES:
             logger.info("Server returned %d, retrying in %d sec.",
                         response.status, RETRY_TIMEOUT)
             await response.release()
             await asyncio.sleep(RETRY_TIMEOUT)
-            return (await self.api_call(method, **params))
+            return await self.api_call(method, **params)
         else:
             if response.headers['content-type'] == 'application/json':
                 err_msg = (await response.json())["description"]
@@ -220,7 +266,25 @@ class Bot:
             logger.error(err_msg)
             raise RuntimeError(err_msg)
 
-    _send_message = partialmethod(api_call, "sendMessage")
+    async def get_me(self):
+        """
+        Returns basic information about the bot
+        (see https://core.telegram.org/bots/api#getme)
+        """
+        json_result = await self.api_call("getMe")
+        return json_result["result"]
+
+    async def leave_chat(self, chat_id):
+        """
+        Use this method for your bot to leave a group, supergroup or channel.
+        Returns True on success.
+
+        :param int chat_id: Unique identifier for the target chat \
+            or username of the target supergroup or channel \
+            (in the format @channelusername)
+        """
+        json_result = await self.api_call("leaveChat", chat_id=chat_id)
+        return json_result["result"]
 
     def send_message(self, chat_id, text, **options):
         """
@@ -231,9 +295,7 @@ class Bot:
         :param options: Additional sendMessage options
             (see https://core.telegram.org/bots/api#sendmessage)
         """
-        return self._send_message(chat_id=chat_id, text=text, **options)
-
-    _edit_message_text = partialmethod(api_call, "editMessageText")
+        return self.api_call("sendMessage", chat_id=chat_id, text=text, **options)
 
     def edit_message_text(self, chat_id, message_id, text, **options):
         """
@@ -244,46 +306,109 @@ class Bot:
         :param str text: Text to edit the message to
         :param options: Additional API options
         """
-        return self._edit_message_text(
+        return self.api_call(
+            "editMessageText",
             chat_id=chat_id,
             message_id=message_id,
             text=text,
             **options
         )
 
-    @asyncio.coroutine
-    def get_file(self, file_id):
+    def edit_message_reply_markup(self, chat_id, message_id, reply_markup, **options):
+        """
+        Edit a reply markup of message in a chat
+
+        :param int chat_id: ID of the chat the message to edit is in
+        :param int message_id: ID of the message to edit
+        :param str reply_markup: New inline keyboard markup for the message
+        :param options: Additional API options
+        """
+        return self.api_call(
+            "editMessageReplyMarkup",
+            chat_id=chat_id,
+            message_id=message_id,
+            reply_markup=reply_markup,
+            **options
+        )
+
+    async def get_file(self, file_id):
         """
         Get basic information about a file and prepare it for downloading.
 
         :param int file_id: File identifier to get information about
         :return: File object (see https://core.telegram.org/bots/api#file)
         """
-        json = yield from self.api_call("getFile", file_id=file_id)
+        json = await self.api_call("getFile", file_id=file_id)
         return json["result"]
 
     def download_file(self, file_path, range=None):
         """
-        Dowload a file from Telegram servers
+        Download a file from Telegram servers
         """
         headers = {"range": range} if range else None
         url = "{0}/file/bot{1}/{2}".format(API_URL, self.api_token, file_path)
-        return aiohttp.get(url, headers=headers)
+        return self.session.get(url, headers=headers)
+
+    _get_user_profile_photos = partialmethod(api_call, "getUserProfilePhotos")
+
+    def get_user_profile_photos(self, user_id, **options):
+        """
+        Get a list of profile pictures for a user
+
+        :param int user_id: Unique identifier of the target user
+        :param options: Additional getUserProfilePhotos options (see
+            https://core.telegram.org/bots/api#getuserprofilephotos)
+        """
+        return self._get_user_profile_photos(
+            user_id=str(user_id),
+            **options
+        )
 
     def track(self, message, name="Message"):
         """
         Track message using http://botan.io
-        Set botak_token to make it work
+        Set botan_token to make it work
         """
         if self.botan_token:
-            asyncio.async(self._track(message, name))
+            asyncio.ensure_future(self._track(message, name))
 
     def stop(self):
         self._running = False
 
-    @asyncio.coroutine
-    def _track(self, message, name):
-        response = yield from aiohttp.post(
+    async def webhook_handle(self, request):
+        """
+        aiohttp.web handle for processing web hooks
+
+        :Example:
+
+        >>> from aiohttp import web
+        >>> app = web.Application()
+        >>> app.router.add_route('/webhook')
+        """
+        update = await request.json()
+        self._process_update(update)
+        return web.Response()
+
+    def create_webhook_app(self, path, loop=None):
+        """
+        Shorthand for creating aiohttp.web.Application with registered webhook hanlde
+        """
+        app = web.Application(loop=loop)
+        app.router.add_route('POST', path, self.webhook_handle)
+        return app
+
+    def set_webhook(self, webhook_url, **options):
+        """
+        Register you webhook url for Telegram service.
+        """
+        return self.api_call(
+            'setWebhook',
+            url=webhook_url,
+            **options
+        )
+
+    async def _track(self, message, name):
+        response = await self.session.post(
             BOTAN_URL,
             params={
                 "token": self.botan_token,
@@ -295,7 +420,7 @@ class Bot:
         )
         if response.status != 200:
             logger.info("error submiting stats %d", response.status)
-        yield from response.release()
+        await response.release()
 
     def _process_message(self, message):
         chat = Chat.from_message(self, message)
@@ -334,19 +459,22 @@ class Bot:
             return
 
         for update in updates["result"]:
-            logger.debug("update %s", update)
-            self._offset = max(self._offset, update["update_id"])
-            coro = None
+            self._process_update(update)
 
-            if "message" in update:
-                coro = self._process_message(update["message"])
-            elif "inline_query" in update:
-                coro = self._process_inline_query(update["inline_query"])
-            elif "callback_query" in update:
-                coro = self._process_callback_query(update["callback_query"])
+    def _process_update(self, update):
+        logger.debug("update %s", update)
+        self._offset = max(self._offset, update["update_id"])
+        coro = None
 
-            if coro:
-                asyncio.async(coro)
+        if "message" in update:
+            coro = self._process_message(update["message"])
+        elif "inline_query" in update:
+            coro = self._process_inline_query(update["inline_query"])
+        elif "callback_query" in update:
+            coro = self._process_callback_query(update["callback_query"])
+
+        if coro:
+            asyncio.ensure_future(coro)
 
 
 class TgBot(Bot):
@@ -388,8 +516,9 @@ class CallbackQuery:
         self.query_id = src['id']
         self.data = src['data']
 
-    def answer(self):
+    def answer(self, **options):
         return self.bot.api_call(
             "answerCallbackQuery",
-            callback_query_id=self.query_id
+            callback_query_id=self.query_id,
+            **options
         )
