@@ -56,6 +56,19 @@ class Chat:
             parse_mode=parse_mode
         )
 
+    def edit_reply_markup(self, message_id, markup):
+        """
+        Edit only reply markup of the message in this chat.
+
+        :param int message_id: ID of the message to edit
+        :param dict markup: Markup options
+        """
+        return self.bot.edit_message_reply_markup(
+            self.id,
+            message_id,
+            reply_markup=json.dumps(markup)
+        )
+
     def _send_to_chat(self, method, **options):
         return self.bot.api_call(
             method,
@@ -63,6 +76,45 @@ class Chat:
             **options
         )
 
+    def get_chat(self):
+        """
+        Get information about the chat.
+        """
+        return self.bot.api_call(
+            "getChat",
+            chat_id=str(self.id)
+        )
+
+    def get_chat_administrators(self):
+        """
+        Get a list of administrators in a chat. Chat must not be private.
+        """
+        return self.bot.api_call(
+            "getChatAdministrators",
+            chat_id=str(self.id)
+        )
+
+    def get_chat_members_count(self):
+        """
+        Get the number of members in a chat.
+        """
+        return self.bot.api_call(
+            "getChatMembersCount",
+            chat_id=str(self.id)
+        )
+
+    def get_chat_member(self, user_id):
+        """
+        Get information about a member of a chat.
+
+        :param int user_id: Unique identifier of the target user
+        """
+        return self.bot.api_call(
+            "getChatMember",
+            chat_id=str(self.id),
+            user_id=str(user_id)
+        )
+
     send_sticker = partialmethod(_send_to_chat, "sendSticker")
 
     def send_audio(self, audio, **options):
@@ -76,7 +128,7 @@ class Chat:
         :Example:
 
         >>> with open("foo.mp3", "rb") as f:
-        >>>     chat.send_audio(f, performer="Bar Fighters", title="Eversong")
+        >>>     await chat.send_audio(f, performer="Foo", title="Eversong")
         """
         return self.bot.api_call(
             "sendAudio",
@@ -97,7 +149,7 @@ class Chat:
         :Example:
 
         >>> with open("foo.png", "rb") as f:
-        >>>     chat.send_photo(f, caption="Would you look at this!")
+        >>>     await chat.send_photo(f, caption="Would you look at this!")
         """
         return self.bot.api_call(
             "sendPhoto",
@@ -119,7 +171,7 @@ class Chat:
         :Example:
 
         >>> with open("foo.mp4", "rb") as f:
-        >>>     chat.send_video(f)
+        >>>     await chat.send_video(f)
         """
         return self.bot.api_call(
             "sendVideo",
@@ -141,7 +193,7 @@ class Chat:
         :Example:
 
         >>> with open("file.doc", "rb") as f:
-        >>>     chat.send_document(f)
+        >>>     await chat.send_document(f)
         """
         return self.bot.api_call(
             "sendDocument",
@@ -162,7 +214,7 @@ class Chat:
         :Example:
 
         >>> with open("voice.ogg", "rb") as f:
-        >>>     chat.send_voice(f)
+        >>>     await chat.send_voice(f)
         """
         return self.bot.api_call(
             "sendVoice",
@@ -188,6 +240,44 @@ class Chat:
             **options
         )
 
+    def send_venue(self, latitude, longitude, title, address, **options):
+        """
+        Send information about a venue.
+
+        :param float latitude: Latitude of the location
+        :param float longitude: Longitude of the location
+        :param str title: Name of the venue
+        :param str address: Address of the venue
+        :param options: Additional sendVenue options (see
+            https://core.telegram.org/bots/api#sendvenue)
+        """
+        return self.bot.api_call(
+            "sendVenue",
+            chat_id=self.id,
+            latitude=latitude,
+            longitude=longitude,
+            title=title,
+            address=address,
+            **options
+        )
+
+    def send_contact(self, phone_number, first_name, **options):
+        """
+        Send phone contacts.
+
+        :param str phone_number: Contact's phone number
+        :param str first_name: Contact's first name
+        :param options: Additional sendContact options (see
+            https://core.telegram.org/bots/api#sendcontact)
+        """
+        return self.bot.api_call(
+            "sendContact",
+            chat_id=self.id,
+            phone_number=phone_number,
+            first_name=first_name,
+            **options
+        )
+
     def send_chat_action(self, action):
         """
         Send a chat action, to tell the user that something is happening on the
@@ -224,6 +314,32 @@ class Chat:
             message_id=message_id
         )
 
+    def kick_chat_member(self, user_id):
+        """
+        Use this method to kick a user from a group or a supergroup.
+        The bot must be an administrator in the group for this to work.
+
+        :param int user_id: Unique identifier of the target user
+        """
+        return self.bot.api_call(
+            "kickChatMember",
+            chat_id=self.id,
+            user_id=user_id
+        )
+
+    def unban_chat_member(self, user_id):
+        """
+        Use this method to unban a previously kicked user in a supergroup.
+        The bot must be an administrator in the group for this to work.
+
+        :param int user_id: Unique identifier of the target user
+        """
+        return self.bot.api_call(
+            "unbanChatMember",
+            chat_id=self.id,
+            user_id=user_id
+        )
+
     def is_group(self):
         """
         Check if this chat is a group.
